import pytest
from absql import Runner as r


@pytest.fixture
def simple_yml_path():
    return "tests/files/simple.yml"


@pytest.fixture
def simple_js_path():
    return "tests/files/simple.js"


@pytest.fixture
def simple_py_path():
    return "tests/files/simple.py"


def test_render_file_return_dict(simple_yml_path):
    file_as_dict = r.render_file(simple_yml_path, return_dict=True)
    print(file_as_dict)
    assert file_as_dict["my_table_placeholder"] == "my_table"
    assert file_as_dict["sql"] == "SELECT * FROM my_table"
    assert file_as_dict["absql_body"] == "SELECT * FROM my_table"


def test_runner_return_dict(simple_yml_path):
    runner = r()
    file_as_dict = runner.render(simple_yml_path, return_dict=True)
    assert file_as_dict["my_table_placeholder"] == "my_table"
    assert file_as_dict["sql"] == "SELECT * FROM my_table"
    assert file_as_dict["absql_body"] == "SELECT * FROM my_table"


def test_render_file_return_dict_js(simple_js_path):
    file_as_dict = r.render_file(simple_js_path, return_dict=True)
    assert file_as_dict["foo"] == "bar"
    assert file_as_dict["absql_body"] == "console.log('hello')"


def test_render_file_return_dict_py(simple_py_path):
    file_as_dict = r.render_file(simple_py_path, return_dict=True)
    print(file_as_dict)
    assert file_as_dict["foo"] == "bar"
    assert file_as_dict["absql_body"] == 'print("hello")'



import pytest
from absql import Runner
from pendulum import now
from datetime import timedelta


@pytest.fixture
def runner():
    return Runner()


@pytest.fixture
def yesterdate():
    return (now(tz="utc") - timedelta(days=1)).to_date_string()


@pytest.fixture
def yesterhour():
    return (
        (now(tz="utc") - timedelta(hours=1, days=0))
        .replace(minute=0, second=0)
        .to_datetime_string()
    )


def test_previous_date(runner, yesterdate):
    assert runner.render("{{previous_date()}}") == yesterdate


def test_previous_hour(runner, yesterhour):
    assert runner.render("{{previous_hour()}}") == yesterhour



import os
import mock
import pytest
from pandas import DataFrame
from sqlalchemy import create_engine
from absql.functions.db import table_exists, query_db, get_max_value, get_min_value
from absql import Runner


@pytest.fixture(scope="module")
def engine():
    engine = create_engine("duckdb:///:memory:")

    df = DataFrame.from_dict(
        {
            "name": ["Thelma", "Bonnie"],
            "friend": ["Louise", "Clyde"],
            "empty": [None, None],
        }
    )

    with engine.connect() as connection:
        with connection.begin():
            df.to_sql("my_table", connection)

    return engine


@pytest.mark.filterwarnings("ignore:Dialect postgresql")
@pytest.mark.filterwarnings("ignore:The Engine.has_table")
@pytest.mark.filterwarnings("ignore:The URL")
def test_table_exists(engine):
    exists_true = table_exists("my_table", engine=engine)
    exists_false = table_exists("nonexistent_table", engine=engine)
    assert exists_true
    assert exists_false is False


@pytest.fixture(autouse=True)
def mock_settings_env_vars():
    with mock.patch.dict(os.environ, {"AB__URI": "sqlite:///:memory:"}):
        yield


@pytest.mark.filterwarnings("ignore:The Engine.has_table")
def test_ab_uri():
    exists_false = table_exists("some_table")
    assert exists_false is False


def test_query_db(engine):
    res = query_db(
        "SELECT name, friend FROM my_table WHERE friend = 'Clyde'", engine=engine
    )
    assert len(res) == 1
    assert res[0].name == "Bonnie"
    assert res[0].friend == "Clyde"


def test_db_functions_in_runner(engine):
    runner = Runner(engine=engine)
    got = runner.render(
        "{{query_db('SELECT COUNT(*) n FROM my_table') | first | attr('n')}}"
    )
    want = "2"
    assert got == want


def test_get_max_value(engine):
    got = get_max_value("my_table.name", engine=engine)
    want = "Thelma"
    assert got == want


def test_get_max_value_null(engine):
    got = get_max_value("my_table.empty", engine=engine)
    want = None
    assert got == want


def test_get_min_value_null(engine):
    got = get_min_value("my_table.name", engine=engine)
    want = "Bonnie"
    assert got == want


def test_get_minmax_value_no_table(engine):
    got = get_min_value("nonexistent_table.nonexistent_column", engine=engine)
    want = None
    assert got == want
    got = get_max_value("nonexistent_table.nonexistent_column", engine=engine)
    assert got == want



import os
import mock
import pytest
from absql import Runner


@pytest.fixture(autouse=True)
def mock_settings_env_vars():
    with mock.patch.dict(os.environ, {"ENV": "test"}):
        yield


@pytest.fixture
def runner():
    return Runner()


def test_env_var(runner):
    got = runner.render("{{env_var('ENV')}}")
    want = "test"
    assert got == want


def test_env_switch(runner):
    got = runner.render("{{env_switch(default='my_default', test='my_test')}}")
    want = "my_test"
    assert got == want


def test_env_switch_default(runner):
    got = runner.render("{{env_switch(default='my_default', prod='my_prod')}}")
    want = "my_default"
    assert got == want


def test_env_switch_unspecified(runner):
    got = runner.render("{{env_switch(prod='my_prod')}}")
    want = "value_unspecified"
    assert got == want



import pytest
from absql.files import parse


@pytest.fixture
def simple_sql_path():
    return "tests/files/simple.sql"


def test_simple_sql(simple_sql_path):
    res = parse(simple_sql_path)
    assert "absql_body" in res.keys()
    assert "my_table_placeholder" in res.keys()
    assert res["my_table_placeholder"] == "my_table"
    assert res["absql_body"] == "SELECT * FROM {{my_table_placeholder}}"


@pytest.mark.parametrize(
    argnames="yml_path",
    argvalues=[
        pytest.param("tests/files/simple.yml", id="simple_yml"),
        pytest.param(
            "tests/files/simple_with_doc_header.yml", id="simple_yml_with_doc_header"
        ),
    ],
)
def test_simple_yml(yml_path):
    res = parse(yml_path)
    assert "sql" in res.keys()
    assert "my_table_placeholder" in res.keys()
    assert res["my_table_placeholder"] == "my_table"
    assert res["sql"] == "SELECT * FROM {{my_table_placeholder}}"



import re
import yaml
import jupytext
from absql.files.loader import generate_loader

FM_BOUNDARY = re.compile(r"^-{3,}\s*$", re.MULTILINE)


def frontmatter_load(file_path, loader=None):
    """
    Loads YAML frontmatter. Expects a YAML block at the top of the file
    that starts and ends with "---" (for non-YAML files). In use in favor of
    frontmatter.load so that custom dag_constructors (via PyYaml) can be used uniformly
    across all file types.
    """
    if loader is None:
        loader = generate_loader()
    with open(file_path, "r") as file:
        text = "".join(file.readlines())
        # Valid YAML files can begin with a document header (i.e. '---') and doesn't
        # require a terminating marker; therefore, the "frontmatter" doesn't necessarily
        # need to begin and end with this string.
        if text.startswith("---") and not file_path.endswith((".yml", ".yaml")):
            _, metadata, content = FM_BOUNDARY.split(text, 2)
            metadata = yaml.load(metadata, Loader=loader)
            content = content.strip("\n")
        elif text.startswith("{"):
            tmp_header = "/*ABSQLQSBA*/ "
            text = tmp_header + text
            metadata = {}
            content = yaml.load(text, Loader=loader)
            content = content.replace(tmp_header, "")
        elif text.startswith("/*") and file_path.endswith(".sql"):
            # Retrieve the first matched set of text within a block comment
            # (i.e. /* ... */).
            metadata = (
                re.compile(r"^\/\*([\S\s]*?)\*\/$", re.MULTILINE).match(text).group(1)
            )
            # Text after the first block-comment end is considered the content of the
            # SQL file. This will handle block comments within the contents as well.
            _, _, content = text.partition("*/")

            metadata = yaml.load(metadata, Loader=loader)
            content = content.strip("\n")
        else:
            metadata = {}
            content = yaml.load(text, Loader=loader)
    return {"metadata": metadata, "content": content}


def parse_yml(file_path, loader=None):
    if loader is None:
        loader = generate_loader()
    raw_content = frontmatter_load(file_path, loader=loader)
    file_content = raw_content["metadata"] or raw_content["content"]
    file_content["absql_body"] = file_content.get("sql", "")
    return file_content


def parse_sql(file_path, loader=None):
    if loader is None:
        loader = generate_loader()
    raw_content = frontmatter_load(file_path, loader=loader)
    file_content = raw_content["metadata"]
    file_content["absql_body"] = raw_content["content"]
    return file_content


def parse_js(file_path, loader=None):
    if loader is None:
        loader = generate_loader()
    with open(file_path, "r") as file:
        text = "".join(file.readlines())
        if text.startswith("/*"):
            # Retrieve the first matched set of text within a block comment
            # (i.e. /* ... */).
            metadata = (
                re.compile(r"^\/\*([\S\s]*?)\*\/$", re.MULTILINE).match(text).group(1)
            )
            # Text after the first block-comment end is considered the content of the
            # SQL file. This will handle block comments within the contents as well.
            _, _, content = text.partition("*/")
            metadata = yaml.load(metadata, Loader=loader)
            content = content.strip("\n")
        else:
            metadata = {}
            content = text.strip("\n")
    file_content = metadata
    file_content["absql_body"] = content
    return file_content


def parse_py(file_path, loader=None):
    if loader is None:
        loader = generate_loader()
    raw_content = jupytext.read(file_path)["cells"]
    if raw_content[0]["source"].startswith("---"):
        file_content = yaml.load(
            raw_content[0]["source"].replace("---", ""), Loader=loader
        )
        file_content["absql_body"] = raw_content[1]["source"]
    else:
        file_content = {"absql_body": raw_content[0]["source"]}
    return file_content
